// Global variables
var bundle = null; // String bundle for localized strings
var applicationLabel = new String;
var applicationCommand = new String;

// Preferences observer object; implements nsIObserver
function dwPrefObserver()
{
  try {
    var pbi = pref.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
    pbi.addObserver(this.domain, this, false);
  } catch(ex) {
    dump("downloadwith: Failed to observe prefs: " + ex + "\n");
  }
}
dwPrefObserver.prototype =
{
  domain: "downloadwith.",
  observe: function(subject, topic, prefName)
  {
    // when downloadwith pref was changed, we reinitialize 
    try{
      downloadwithPrefInit();
    } catch(err){}
  }
}  


// this code is performed when the script is loaded:
// get the preferences service
var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
var pref = prefService.getBranch(null);
var dwPref = prefService.getBranch("downloadwith.");
window.downloadwithPrefObserver = new dwPrefObserver();


// do initialisation stuff, like read the preferences
function downloadwithPrefInit()
{ 
	// load preferences into JS variables, or restore default prefs if none are found
	dump("downloadwith: init\n");
	window.removeEventListener("load", downloadwithPrefInit, true);

	// read prefs or set defaults if no prefs exist
	try
	{
		downloadwithReadPref();  
		bundle = document.getElementById("downloadwithbundle");
	}
	catch(e)
	{
		dump("downloadwith: Setting Default Preferences\n");
		dwPref.setCharPref("applicationLabel", "GetRight");
		dwPref.setCharPref("applicationCommand", "C:\\Arquivos de programas\\GetRight\\getright.exe");
		downloadwithReadPref();
	}  
}



function downloadwithReadPref()
{
	applicationLabel = dwPref.getCharPref("applicationLabel");
	applicationCommand = dwPref.getCharPref("applicationCommand");
}


var DownloadWith = {
  init: function() {
    document.getElementById("contentAreaContextMenu").addEventListener("popupshowing",DownloadWith.hideMenu,false);
  },

  commanded: function(e,menu) {
    var t = e.target;
    if(t.id != "downloadwith-invalide-url") {
      // load url
      var baseurl = t.getAttribute("label");
      if(/http:\/\//.test(baseurl)) {
        baseurl = "http:\/\/" + baseurl.split("http:\/\/")[1];
      } else if (/ftp:\/\//.test(baseurl)) {
        baseurl = "ftp:\/\/" + baseurl.split("ftp:\/\/")[1];
      } 
      // create a file object for the external program

      var theapp = Components.classes["@mozilla.org/filespec;1"].createInstance(Components.interfaces.nsIFileSpec);
	theapp.nativePath  = applicationCommand;
      args = " " + baseurl;
      theapp.execute( args );
    }
    if(e.button==1) menu.hidePopup();
  },

  // called onPopupShowing for the main context menu. hides DownloadWith if it's not relevant
  hideMenu: function() {
    var cm = gContextMenu;
    document.getElementById("context-downloadwith").hidden = !cm.onLink;
  },

  clearMenu: function(menu) {
    while(menu.hasChildNodes()) menu.removeChild(menu.lastChild);
  },

  // wander out from the node the context menu was called on and try and find a link
  // return the href attribute.
  // in future this may need expanding to deal with xlinks, but we'll live without for now
  getLink: function() {
    var elem = document.popupNode;
    while(elem) {
      if(elem instanceof Components.interfaces.nsIDOMHTMLAnchorElement && elem.href)
        return elem.href;
      elem = elem.parentNode;
    }
    return null;
  },

  buildMenu: function(menu) {
    var url = this.getLink();
    if(!url) return;
    this.addItem(menu, url);   
  },

  addSeperator: function(menu) {
    menu.appendChild(document.createElement("menuseparator"));
  },
  insertSeperatorAfter: function(menuitem) {
    var separator = document.createElement("menuseparator");
    menuitem.parentNode.insertBefore(separator, menuitem.nextSibling);
  },
  addItem: function(menu, url) {
    var menuitem = document.createElement("menuitem");
    if(/^(ht|f)tp:\/\//.test(url)) {
      menuitem.setAttribute("label",applicationLabel  + " " + url);
    } else {
      menuitem.setAttribute("label",bundle.getString("invalideURL"));
      menuitem.setAttribute("id","downloadwith-invalide-url");
    }
    menu.appendChild(menuitem);
  }
};

window.addEventListener("load",downloadwithPrefInit,true);
window.addEventListener("load",DownloadWith.init,false);